%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Funao que calcula o centroide e as percentagens de energia aos motores com base na imagem binaria
% [bimageout percentageme percentagemd caso]=controlo(bimage,margens,centro,props)
% Entradas: 
% bimage--> Imagem binaria
% margens--> impoe margens  imagem para limitar o tamanho da imagem no
% calculo da energia aos motores - permite aumentar a velocidade de
% resposta a desvios
% centro--> correspnde a percentagem do tamanho da imagem na direco
% vertical em que se pretende colocar a referencia de controlo
% props--> percenteagem (importancia) que se pretende atribuir ao controlo
% horizontal e vertical
% Saidas: 
% bimageout--> Imagem binaria com um quadrado no centroide ()
% percentageme e percentagemd--> Percentagens de energia aos motores esq e dto
% caso--> o controlo foi dividido em casos dependendo da localizao do
% centroide para depois fazer a actuao indicada aos motores
%Builded by Gonalo Madeira e Ins Faria Out2004
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [bimageout percentageme percentagemd caso]=controlo(bimage,margens,centro,props)
tamanho=size(bimage);          

%ciclos que calculam o centroide
contx=0;
conty=0; 
cont=0;
for i=1:tamanho(1),
    for j=1:tamanho(2),
        if bimage(i,j)==0,
            contx=contx+j/1000; %divide-se por 1000 para o numero nao ser exageradamente grande
            conty=conty+i/1000;
            cont=cont+1/1000;
        end
    end
end
if cont==0,
    iXcm=uint16(tamanho(2)/2);
    iYcm=uint16(tamanho(1)/2);
    Xcm=(tamanho(2)/2);
    Ycm=(tamanho(1)/2);
    bimageout=bimage;
    bimageout(iYcm-1:iYcm+1,iXcm-1:iXcm+1)=0;
else
    Xcm=(contx/cont);
    Ycm=(conty/cont);
    iXcm=uint16(Xcm);
    iYcm=uint16(Ycm);

    %cria figura com um quadrado no centro de massa branco ou preto de
    %acordo com o fundo de maneira a ver-se o centroide
    bimageout=bimage;
    if bimageout(iYcm,iXcm)==1;
        bimageout(iYcm-1:iYcm+1,iXcm-1:iXcm+1)=0;
    else         
        bimageout(iYcm-1:iYcm+1,iXcm-1:iXcm+1)=1;
    end
end

%Parametros variaveis introduzidos pelo utilizador
propx=props(1);  %percentagens dadas  importancia do controlo horizontal e vertical
propy=props(2);  %permite atenuar variacoes bruscas
centroy=centro;
margemx=tamanho(2)*margens(1); %pixeis
margemy=tamanho(1)*margens(2); %pixeis

% Aqui apresentam-se varias hipoteses de funcoes de controlo para estimar a
% energia aos motores

%%%%%%%CONTROLO PROPORCIONAL PARABOLICO  CONCAVIDADE PARA CIMA%%%%%%%%%%%%% 

% Percentagem da energia aos motores devido ao desvio em x-------------
% if Xcm>margemx && Xcm<tamanho(2)-margemx
%     Zx=(((Xcm-tamanho(2)/2))^2)/((margemx-tamanho(2)/2)^2);
% 
% elseif Xcm<=margemx || Xcm>=tamanho(2)-margemx
%     Zx=1;
% end

% %Percentagem da energia aos motores devido ao desvio em y--------------
% if  Ycm>margemy && Ycm<=centroy*tamanho(1)
%     Zy=(Ycm-centroy*tamanho(1))^2/(margemy-centroy*tamanho(1))^2;
%     %Zy=sqrt(1-(Ycm-margemy)^2/(0.75*tamanho(1)-margemy)^2);
% elseif Ycm<=margemy
%     Zy=1;
% else
%     Zy=0;
% end


%%%%%%%CONTROLO PROPORCIONAL PARABOLICO  CONCAVIDADE PARA BAIXO%%%%%%%%%%%%
% if Xcm>margemx && Xcm<tamanho(2)/2
%     Zx=sqrt((tamanho(2)/2-Xcm)/(tamanho(2)/2-margemx));
% elseif Xcm<tamanho(2)-margemx && Xcm>tamanho(2)/2
%     Zx=sqrt((Xcm-tamanho(2)/2)/(tamanho(2)*0.5-margemx));
% elseif Xcm<=margemx || Xcm>=tamanho(2)-margemx
%     Zx=1;
% end

%Percentagem da energia aos motores devido ao desvio em y--------------
% if  Ycm>margemy && Ycm<=centroy*tamanho(1)
%     Zy=sqrt((centroy*tamanho(1)-Ycm)/(centroy*tamanho(1)-margemy));
%  elseif Ycm<=margemy
%     Zy=1;
% else
%     Zy=0;
% end



%%%%%%%CONTROLO PROPORCIONAL LINEAR%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% %Percentagem da energia aos motores devido ao desvio em x-------------
if Xcm>margemx && Xcm<tamanho(2)/2
    Zx=(tamanho(2)/2-Xcm)/(tamanho(2)/2-margemx);
elseif Xcm<tamanho(2)-margemx && Xcm>tamanho(2)/2
    Zx=(Xcm-tamanho(2)/2)/(tamanho(2)*0.5-margemx);
elseif Xcm<=margemx || Xcm>=tamanho(2)-margemx
    Zx=1;
end
% 
%Percentagem da energia aos motores devido ao desvio em y--------------
if  Ycm>margemy && Ycm<=centroy*tamanho(1)
    Zy=(centroy*tamanho(1)-Ycm)/(centroy*tamanho(1)-margemy);
 elseif Ycm<=margemy
    Zy=1;
else
    Zy=0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%o desvio segundo x e mais importante que o segundo y (se a curva for muito
%apertada ele tem de abrandar para virar =))
%desacelerao para fazer curvas
if Zx+Zy>1
    Zy=1-Zx;
end
%-------------------------
raio=tamanho(2)*0.2;

%valores para os motores esq e dto de acordo com a localizao do centroide
if Xcm>tamanho(2)/2 && Ycm<=centroy*tamanho(1);
    percentageme=Zx*propx+Zy*propy;
    percentagemd=Zy*propy;
    caso=1;
    
elseif Xcm<=tamanho(2)/2 && Ycm<=centroy*tamanho(1)
    percentageme=Zy*propy;
    percentagemd=Zx*propx+Zy*propy;
    caso=1;
    
elseif Xcm>=tamanho(2)/2+raio && Ycm>centroy*tamanho(1) %Ycm>-sqrt(raio^2-(Xcm-tamanho(2))^2+tamanho(1))
    percentageme=Zx*propx;
    percentagemd=0;
    caso=2;
    
elseif Xcm<=tamanho(2)/2-raio && Ycm>centroy*tamanho(1) %&& Ycm>-sqrt(raio^2-(Xcm-tamanho(2))^2+tamanho(1))
    percentageme=0;
    percentagemd=Zx*propx; 
    caso=3;
    
elseif Ycm>centroy*tamanho(1) && Xcm>tamanho(2)/2-raio && Xcm<tamanho(2)/2+raio %&& Ycm<-sqrt(raio^2-(Xcm-tamanho(2))^2+tamanho(1))
    percentageme=0;
    percentagemd=0;
    caso=4;
end

%teste=[Zx Zy]

